(*-----------------------------------------------------------------------*)
(* ECCANOM: calculation of the eccentric anomaly E=ECCANOM(MAN,ECC)      *)
(*          from the mean anomaly MAN and the eccentricity ECC.          *)
(*          (solution of Kepler's equation by Newton's method)           *)
(*          (E, MAN in degrees)                                          *)
(*-----------------------------------------------------------------------*)
FUNCTION ECCANOM(MAN,ECC:extended):extended;
  CONST PI=3.141592654; TWOPI=6.283185308; RAD=0.0174532925199433;
        EPS = 1E-11; MAXIT = 15;
  VAR M,E,F: extended;
      I    : INTEGER;
  BEGIN
    M:=MAN/360.0;  M:=TWOPI*(M-TRUNC(M)); IF M<0 THEN M:=M+TWOPI;
    IF (ECC<0.8) THEN E:=M ELSE E:=PI;
    F := E - ECC*SIN(E) - M; I:=0;
    WHILE ( (ABS(F)>EPS) AND (I<MAXIT) ) DO
      BEGIN
        E := E - F / (1.0-ECC*COS(E));  F := E-ECC*SIN(E)-M; I:=I+1;
      END;
    ECCANOM:=E/RAD;
    IF (I=MAXIT) THEN  WRITELN(' convergence problems in ECCANOM');
  END;

(*-----------------------------------------------------------------------*)
(* ELEMENT: calculates orbital elements from two positions               *)
(*          for elliptic and hyperbolic orbits                           *)
(*                                                                       *)
(*   JDA,JDB: time of passage of points A and B (Julian Date)            *)
(*   RA, RB : position vectors of points A and B                         *)
(*   TP     : perihelion time (in Julian centuries since J2000)          *)
(*   Q      : perihelion distance                                        *)
(*   ECC    : eccentricity                                               *)
(*   INC    : inclination (in deg)                                       *)
(*   LAN    : longitude of the ascending node (in deg)                   *)
(*   AOP    : argument of perihelion (in deg)                            *)
(*-----------------------------------------------------------------------*)
PROCEDURE ELEMENT ( JDA,JDB: extended; RA,RB: VECTOR;
                    VAR TP,Q,ECC,INC,LAN,AOP: extended);

  CONST KGAUSS = 0.01720209895;
        RAD    = 0.01745329252; (* 180/pi *)

  VAR   TAU,ETA,P,AX,N,NY,E,M,U         : extended;
        SA,SB,S0,FAC,DUMMY,SHH          : extended;
        COS_DNY,SIN_DNY,ECOS_NY,ESIN_NY : extended;
        EA,R0,E0,R                      : VECTOR;
        I                               : INDEX;

  BEGIN

    (* calculate vector R0 (fraction of RB perpendicular to RA)   *)
    (* and the magnitudes of RA, RB and R0                        *)
    SA := NORM(RA);     FOR I:=X TO Z DO  EA[I]:=RA[I]/SA;
    SB := NORM(RB);
    FAC := DOT(RB,EA);  FOR I:=X TO Z DO  R0[I]:=RB[I]-FAC*EA[I];
    S0 := NORM(R0);     FOR I:=X TO Z DO  E0[I]:=R0[I]/S0;

    (* inclination and ascending node *)
    CROSS (EA,E0,R);
    POLAR ( -R[Y],R[X],R[Z], DUMMY,INC,LAN );  INC := 90.0-INC;
    U  :=  ATN2 ( (+E0[X]*R[Y]-E0[Y]*R[X]) , (-EA[X]*R[Y]+EA[Y]*R[X]) );
    IF INC=0.0 THEN U:=ATN2(RA[Y],RA[X]);

    (* semilatus rectum p *)
    TAU := KGAUSS * ABS(JDB-JDA);   ETA := FIND_ETA(RA,RB,TAU);
    P := SA*S0*ETA / TAU;   P := P*P;

    (* eccentricity, true anomaly and longitude of perihelion *)
    COS_DNY := FAC/SB;    SIN_DNY := S0/SB;
    ECOS_NY := P/SA-1.0;  ESIN_NY := (ECOS_NY*COS_DNY-(P/SB-1.0))/SIN_DNY;
    POLAR ( ECOS_NY,ESIN_NY,0.0, ECC,DUMMY,NY );
    AOP := U-NY;  WHILE (AOP<0.0) DO AOP:=AOP+360.0;

    (* perihelion distance, semimajor axis and mean daily motion *)
    Q  := P/(1.0+ECC);     AX := Q/(1.0-ECC);
    N  := KGAUSS / SQRT(ABS(AX*AX*AX));

    (* mean anomaly and time of perihelion passage *)
    IF (ECC<1.0)
      THEN
        BEGIN
          E := ATN2 ( SQRT((1.0-ECC)*(1.0+ECC))*ESIN_NY, ECOS_NY+ECC*ECC );
          E := RAD*E;  M := E-ECC*SIN(E); ;
        END
      ELSE
        BEGIN
          SHH := SQRT((ECC-1.0)*(ECC+1.0))*ESIN_NY / (ECC+ECC*ECOS_NY) ;
          M   := ECC*SHH - LN(SHH+SQRT(1.0+SHH*SHH))
        END;
    TP := ( (JDA-M/N) - 2451545.0 ) / 36525.0;

  END;

(*-----------------------------------------------------------------------*)
(* ELLIP: calculation of position and velocity vector                    *)
(*        for elliptic orbits                                            *)
(*                                                                       *)
(*        M    mean anomaly in degrees       X,Y    position   (in AU)   *)
(*        A    semimajor axis                VX,VY  velocity (in AU/day) *)
(*        ECC  eccentricity                                              *)
(*-----------------------------------------------------------------------*)
PROCEDURE ELLIP(M,A,ECC:extended;VAR X,Y,VX,VY:extended);
  CONST KGAUSS = 0.01720209895;
  VAR K,E,C,S,FAC,RHO: extended;
  BEGIN
    K  := KGAUSS / SQRT(A);
    E  := ECCANOM(M,ECC);   C:=CS(E); S:=SN(E);
    FAC:= SQRT((1.0-ECC)*(1+ECC));  RHO:=1.0-ECC*C;
    X := A*(C-ECC); Y :=A*FAC*S;   VX:=-K*S/RHO;   VY:=K*FAC*C/RHO;
  END;

(*-----------------------------------------------------------------------*)
(* FIND_ETA: determines the sector/triangle ratio                        *)
(*           from two positions and the time difference                  *)
(*-----------------------------------------------------------------------*)
FUNCTION FIND_ETA ( RA,RB: VECTOR; TAU: extended ): extended;

  CONST DELTA = 1.0E-9;  MAXIT = 30;

  VAR   KAPPA,M,L,SA,SB,ETA_MIN,ETA1,ETA2,F1,F2,D_ETA: extended;
        I: INTEGER;

  (* F(eta) = 1 - eta + (m/eta**2)*W(m/eta**2-l) *)
  FUNCTION F ( ETA,M,L: extended ): extended;
    CONST EPS =1.0E-10;
    VAR W,WW,A,S,N,G,E: extended;
    BEGIN
      W := M/(ETA*ETA)-L;
      IF (ABS(W)<0.1)
        THEN  (* series expansion *)
          BEGIN
            A:=4.0/3.0; WW:=A; N:=0.0;
            REPEAT
              N:=N+1; A:=A*W*(N+2.0)/(N+1.5); WW:=WW+A;
            UNTIL ABS(A)<EPS;
          END
        ELSE
          IF (W>0)
            THEN (* W=(2g-sin2g)/(sin(g)**3), g=2*arcsin(sqrt(w)) *)
              BEGIN
                G := 2.0*ARCTAN(SQRT(W/(1.0-W)));  S := SIN(G);
                WW := (2.0*G-SIN(2.0*G))/(S*S*S);
              END
            ELSE (* W=(sinh2g-2g)/(sinh(g)**3), g=2*arsinh(sqrt(-w)) *)
              BEGIN
                G := 2.0*LN(SQRT(-W)+SQRT(1.0-W));
                E := EXP(G); S:=0.5*(E-1.0/E); E:=E*E;
                WW := (0.5*(E-1.0/E)-2.0*G)/(S*S*S);
              END;
       F := 1.0-ETA+(W+L)*WW;
     END; (* FIND_ETA.F *)

  BEGIN

    SA := NORM(RA);  SB := NORM(RB);  KAPPA := SQRT(2.0*(SA*SB+DOT(RA,RB)));
    M := TAU*TAU / (KAPPA*KAPPA*KAPPA);   L := (SA+SB)/(2.0*KAPPA) - 0.5;
    ETA_MIN := SQRT(M/(L+1.0));

    (* start with Hansen's approximation *)
    ETA2 := ( 12.0 + 10.0*SQRT(1.0+(44.0/9.0)*M/(L+5.0/6.0)) ) / 22.0;
    ETA1 := ETA2 + 0.1;   F1 := F(ETA1,M,L);   F2 := F(ETA2,M,L);  I := 0;

    (* secant method *)
    WHILE ( (ABS(F2-F1)>DELTA) AND (I<MAXIT) ) DO
      BEGIN
        D_ETA:=-F2*(ETA2-ETA1)/(F2-F1);  ETA1:=ETA2; F1:=F2;
        WHILE (ETA2+D_ETA<=ETA_MIN) DO D_ETA:=0.5*D_ETA;
        ETA2:=ETA2+D_ETA;  F2:=F(ETA2,M,L);
      END;
    IF (I=MAXIT) THEN  WRITELN(' convergence problems in FIND_ETA');
    FIND_ETA := ETA2;

  END;


(*-----------------------------------------------------------------------*)
(* HYPANOM: calculation of the eccentric anomaly H=HYPANOM(MH,ECC) from  *)
(*          mean anomaly MH and eccentricity ECC for                     *)
(*          hyperbolic orbits                                            *)
(*-----------------------------------------------------------------------*)
FUNCTION HYPANOM(MH,ECC:extended):extended;
  CONST EPS=1E-10; MAXIT=15;
  VAR   H,F,EXH,SINHH,COSHH: extended;
        I                  : INTEGER;
  BEGIN
    H:=LN(2.0*ABS(MH)/ECC+1.8); IF (MH<0.0) THEN H:=-H;
    EXH:=EXP(H); SINHH:=0.5*(EXH-1.0/EXH); COSHH:=0.5*(EXH+1.0/EXH);
    F := ECC*SINHH-H-MH; I:=0;
    WHILE ( (ABS(F)>EPS*(1.0+ABS(H+MH))) AND (I<MAXIT) )  DO
      BEGIN
        H := H - F / (ECC*COSHH-1.0);
        EXH:=EXP(H); SINHH:=0.5*(EXH-1.0/EXH); COSHH:=0.5*(EXH+1.0/EXH);
        F := ECC*SINHH-H-MH; I:=I+1;
      END;
    HYPANOM:=H;
    IF (I=MAXIT) THEN  WRITELN(' convergence problems in HYPANOM');
  END;


(*-----------------------------------------------------------------------*)
(* HYPERB: calculation of the position and velocity vector               *)
(*         for hyperbolic orbits                                         *)
(*                                                                       *)
(*   T0   time of perihelion passage             X,Y   position          *)
(*   T    time                                   VX,VY velocity          *)
(*   A    semimajor axis (arbitrary sign)                                *)
(*   ECC  eccentricity                                                   *)
(*   (T0,T in julian centuries since J2000)                              *)
(*-----------------------------------------------------------------------*)
PROCEDURE HYPERB(T0,T,A,ECC:extended;VAR X,Y,VX,VY:extended);
  CONST KGAUSS = 0.01720209895;
  VAR K,MH,H,EXH,COSHH,SINHH,RHO,FAC: extended;
  BEGIN
    A   := ABS(A);
    K   := KGAUSS / SQRT(A);
    MH  := K*36525.0*(T-T0)/A;
    H   := HYPANOM(MH,ECC);
    EXH := EXP(H);   COSHH:=0.5*(EXH+1.0/EXH);  SINHH:=0.5*(EXH-1.0/EXH);
    FAC := SQRT((ECC+1.0)*(ECC-1.0));   RHO := ECC*COSHH-1.0;
    X   := A*(ECC-COSHH);  Y  := A*FAC*SINHH;
    VX  :=-K*SINHH/RHO;    VY := K*FAC*COSHH/RHO;
  END;

(*-----------------------------------------------------------------------*)
(* KEPLER: calculation of position and velocity for unperturbed          *)
(*         elliptic, parabolic and hyperbolic orbits                     *)
(*                                                                       *)
(*        T0   time of perihelion passage         X,Y,Z     position     *)
(*        T    time                               VX,VY,VZ  velocity     *)
(*        Q    perihelion distance                                       *)
(*        ECC  eccentricity                                              *)
(*        PQR  matrix of Gaussian vectors                                *)
(*        (T0,T in Julian centuries since J2000)                         *)
(*-----------------------------------------------------------------------*)
PROCEDURE KEPLER(T0,T,Q,ECC:extended;PQR:extended33;VAR X,Y,Z,VX,VY,VZ:extended);
  CONST M0=5.0; EPS=0.1;
        KGAUSS = 0.01720209895; DEG = 57.29577951;
  VAR M,DELTA,TAU,INVAX,XX,YY,VVX,VVY: extended;
  BEGIN
    DELTA := ABS(1.0-ECC);
    INVAX := DELTA / Q;
    TAU   := KGAUSS*36525.0*(T-T0);
    M     := DEG*TAU*SQRT(INVAX*INVAX*INVAX);
    IF ( (M<M0) AND (DELTA<EPS) )
      THEN  PARAB(T0,T,Q,ECC,XX,YY,VVX,VVY)
      ELSE IF (ECC<1.0)
             THEN ELLIP (M,1.0/INVAX,ECC,XX,YY,VVX,VVY)
             ELSE HYPERB(T0,T,1.0/INVAX,ECC,XX,YY,VVX,VVY);
    ORBECL(XX,YY,PQR,X,Y,Z); ORBECL(VVX,VVY,PQR,VX,VY,VZ);
  END;

(*-----------------------------------------------------------------------*)
(* PARAB: calculation of position and velocity for                       *)
(*        parabolic and near parabolic orbits according to Stumpff       *)
(*                                                                       *)
(*        T0   time of perihelion passage         X,Y    position        *)
(*        T    time                              VX,VY  velocity         *)
(*        Q    perihelion distance                                       *)
(*        ECC  eccentricity                                              *)
(*        (T0,T in julian centuries since J2000)                         *)
(*-----------------------------------------------------------------------*)
PROCEDURE PARAB(T0,T,Q,ECC:extended;VAR X,Y,VX,VY:extended);
  CONST EPS    = 1E-9;
        KGAUSS = 0.01720209895;
        MAXIT  = 15;
  VAR E2,E20,FAC,C1,C2,C3,K,TAU,A,U,U2: extended;
      R: extended;
      I: INTEGER;
  BEGIN
    E2:=0.0;  FAC:=0.5*ECC;  I:=0;
    K   := KGAUSS / SQRT(Q*(1.0+ECC));
    TAU := KGAUSS * 36525.0*(T-T0);
    REPEAT
      I:=I+1;
      E20:=E2;
      A:=1.5*SQRT(FAC/(Q*Q*Q))*TAU;  A:=CUBR(SQRT(A*A+1.0)+A);
      U:=A-1.0/A;  U2:=U*U;  E2:=U2*(1.0-ECC)/FAC;
      STUMPFF(E2,C1,C2,C3); FAC:=3.0*ECC*C3;
    UNTIL (ABS(E2-E20)<EPS)OR(I>MAXIT);
    IF (I=MAXIT) THEN  WRITELN(' convergence problems in PARAB');
    R :=Q*(1.0+U2*C2*ECC/FAC);
    X :=Q*(1.0-U2*C2/FAC);          VY:= K*(X/R+ECC);
    Y :=Q*SQRT((1.0+ECC)/FAC)*U*C1; VX:=-K*Y/R;
  END;

(*-----------------------------------------------------------------------*)
(*  STUMPFF: calculation of Stumpff's functions C1 = sin(E)/E ,          *)
(*           C2 = (1-cos(E))/(E**2) and C3 = (E-sin(E))/(E**3)           *)
(*           for argument E2=E**2                                        *)
(*           (E: eccentric anomaly in radian)                            *)
(*-----------------------------------------------------------------------*)
PROCEDURE STUMPFF(E2:extended;VAR C1,C2,C3:extended);
  CONST EPS=1E-12;
  VAR N,ADD: extended;
  BEGIN
    C1:=0.0; C2:=0.0; C3:=0.0; ADD:=1.0; N:=1.0;
    REPEAT
      C1:=C1+ADD; ADD:=ADD/(2.0*N);
      C2:=C2+ADD; ADD:=ADD/(2.0*N+1.0);
      C3:=C3+ADD; ADD:=-E2*ADD; N:=N+1.0;
    UNTIL ABS(ADD)<EPS;
  END;

