(*-----------------------------------------------------------------------*)
(* NUTEQU: transformation of mean to true coordinates                    *)
(*         (including terms >0.1" according to IAU 1980)                 *)
(*         T = (JD-2451545.0)/36525.0                                    *)
(*-----------------------------------------------------------------------*)
PROCEDURE NUTEQU(T:extended;VAR X,Y,Z:extended);
  CONST ARC=206264.8062;          (* arcseconds per radian = 3600*180/pi *)
        P2 =6.283185307;          (* 2*pi                                *)
  VAR   LS,D,F,N,EPS : extended;
        DPSI,DEPS,C,S: extended;
        DX,DY,DZ     : extended;
  FUNCTION FRAC(X:extended):extended;
    (* with several compilers it may be necessary to replace TRUNC *)
    (* by LONG_TRUNC or INT if T<-24!                              *)
    BEGIN  FRAC:=X-TRUNC(X) END;
  BEGIN
    LS  := P2*FRAC(0.993133+  99.997306*T); (* mean anomaly Sun          *)
    D   := P2*FRAC(0.827362+1236.853087*T); (* diff. longitude Moon-Sun  *)
    F   := P2*FRAC(0.259089+1342.227826*T); (* mean argument of latitude *)
    N   := P2*FRAC(0.347346-   5.372447*T); (* longit. ascending node    *)
    EPS := 0.4090928-2.2696E-4*T;           (* obliquity of the ecliptic *)
    DPSI := ( -17.200*SIN(N)   - 1.319*SIN(2*(F-D+N)) - 0.227*SIN(2*(F+N))
              + 0.206*SIN(2*N) + 0.143*SIN(LS) ) / ARC;
    DEPS := ( + 9.203*COS(N)   + 0.574*COS(2*(F-D+N)) + 0.098*COS(2*(F+N))
              - 0.090*COS(2*N)                 ) / ARC;
    C := DPSI*COS(EPS);  S := DPSI*SIN(EPS);
    DX := -(C*Y+S*Z); DY := (C*X-DEPS*Z); DZ := (S*X+DEPS*Y);
    X  :=  X + DX;        Y  := Y + DY;       Z  := Z + DZ;
  END;

(*-----------------------------------------------------------------------*)
(* PMATECL: calculates the precession matrix A[i,j] for                  *)
(*          transforming ecliptic coordinates from equinox T1 to T2      *)
(*          ( T=(JD-2451545.0)/36525 )                                   *)
(*-----------------------------------------------------------------------*)
PROCEDURE PMATECL(T1,T2:extended;VAR A: extended33);
  CONST SEC=3600.0;
  VAR DT,PPI,PI,PA: extended;
      C1,S1,C2,S2,C3,S3: extended;
  BEGIN
    DT:=T2-T1;
    PPI := 174.876383889 +( ((3289.4789+0.60622*T1)*T1) +
              ((-869.8089-0.50491*T1) + 0.03536*DT)*DT )/SEC;
    PI  := ( (47.0029-(0.06603-0.000598*T1)*T1)+
             ((-0.03302+0.000598*T1)+0.000060*DT)*DT )*DT/SEC;
    PA  := ( (5029.0966+(2.22226-0.000042*T1)*T1)+
             ((1.11113-0.000042*T1)-0.000006*DT)*DT )*DT/SEC;
    C1:=CS(PPI+PA);  C2:=CS(PI);  C3:=CS(PPI);
    S1:=SN(PPI+PA);  S2:=SN(PI);  S3:=SN(PPI);
    A[1,1]:=+C1*C3+S1*C2*S3; A[1,2]:=+C1*S3-S1*C2*C3; A[1,3]:=-S1*S2;
    A[2,1]:=+S1*C3-C1*C2*S3; A[2,2]:=+S1*S3+C1*C2*C3; A[2,3]:=+C1*S2;
    A[3,1]:=+S2*S3;          A[3,2]:=-S2*C3;          A[3,3]:=+C2;
  END;
(*-----------------------------------------------------------------------*)
(* PMATEQU: calculates the precession matrix A[i,j] for                  *)
(*          transforming equatorial coordinates from equinox T1 to T2    *)
(*          (T=(JD-2451545.0)/36525 )                                    *)
(*-----------------------------------------------------------------------*)
PROCEDURE PMATEQU(T1,T2:extended;VAR A:extended33);
  CONST SEC=3600.0;
  VAR DT,ZETA,Z,THETA: extended;
      C1,S1,C2,S2,C3,S3: extended;
  BEGIN
   DT:=T2-T1;
    ZETA  :=  ( (2306.2181+(1.39656-0.000139*T1)*T1)+
                ((0.30188-0.000345*T1)+0.017998*DT)*DT )*DT/SEC;
    Z     :=  ZETA + ( (0.79280+0.000411*T1)+0.000205*DT)*DT*DT/SEC;
    THETA :=  ( (2004.3109-(0.85330+0.000217*T1)*T1)-
                ((0.42665+0.000217*T1)+0.041833*DT)*DT )*DT/SEC;
    C1:=CS(Z);  C2:=CS(THETA);  C3:=CS(ZETA);
    S1:=SN(Z);  S2:=SN(THETA);  S3:=SN(ZETA);
    A[1,1]:=-S1*S3+C1*C2*C3; A[1,2]:=-S1*C3-C1*C2*S3; A[1,3]:=-C1*S2;
    A[2,1]:=+C1*S3+S1*C2*C3; A[2,2]:=+C1*C3-S1*C2*S3; A[2,3]:=-S1*S2;
    A[3,1]:=+S2*C3;          A[3,2]:=-S2*S3;          A[3,3]:=+C2;
  END;
(*-----------------------------------------------------------------------*)
(* PN_MATRIX: combined precession and nutation matrix for transformation *)
(*            from mean equinox T0 to true equinox T                     *)
(*            T0,T in Julian cent. since J2000; T=(JD-2451545.0)/36525   *)
(*-----------------------------------------------------------------------*)
PROCEDURE PN_MATRIX ( T0,T:extended; VAR A: extended33 );
  BEGIN
    PMATEQU(T0,T,A);                  (* precession matrix T0->T;     *)
    NUTEQU(T,A[1,1],A[2,1],A[3,1]);   (* transform column vectors of  *)
    NUTEQU(T,A[1,2],A[2,2],A[3,2]);   (* matrix A from mean equinox T *)
    NUTEQU(T,A[1,3],A[2,3],A[3,3]);   (* to true equinox T            *)
  END;

(*-----------------------------------------------------------------------*)
(* PRECART: calculate change of coordinates due to precession            *)
(*          for given transformation matrix A[i,j]                       *)
(*          (to be used with PMATECL und PMATEQU)                        *)
(*-----------------------------------------------------------------------*)
PROCEDURE PRECART(A:extended33; VAR X,Y,Z:extended);
  VAR U,V,W: extended;
  BEGIN
    U := A[1,1]*X+A[1,2]*Y+A[1,3]*Z;
    V := A[2,1]*X+A[2,2]*Y+A[2,3]*Z;
    W := A[3,1]*X+A[3,2]*Y+A[3,3]*Z;
    X:=U; Y:=V; Z:=W;
  END;
