(*-----------------------------------------------------------------------*)
(* ABERRAT: velocity vector of the Earth in equatorial coordinates       *)
(*          (in units of the velocity of light)                          *)
(*-----------------------------------------------------------------------*)
PROCEDURE ABERRAT(T: extended; VAR VX,VY,VZ: extended);
  CONST P2=6.283185307;
  VAR L,CL: extended;
  FUNCTION FRAC(X:extended):extended;
    BEGIN  X:=X-TRUNC(X); IF (X<0) THEN X:=X+1; FRAC:=X  END;
  BEGIN
    L := P2*FRAC(0.27908+100.00214*T);  CL:=COS(L); 
    VX := -0.994E-4*SIN(L); VY := +0.912E-4*CL; VZ := +0.395E-4*CL;
  END;

(*-----------------------------------------------------------------------*)
(* APPARENT  : apparent coordinates of a star                            *)
(*   PNMAT   : matrix for precession and nutation                        *)
(*   VX,VY,VZ: velocity of the earth (equatorial coord.; in units of c)  *)
(*   RA,DEC  : right ascension and declination                           *)
(*-----------------------------------------------------------------------*)
PROCEDURE APPARENT ( PNMAT: extended33; VX,VY,VZ: extended; VAR RA,DEC: extended );
  VAR X,Y,Z,R: extended;
  BEGIN
    CART ( 1.0, DEC,RA, X,Y,Z ); (* cartesian coordinates of the star    *)
    PRECART ( PNMAT , X,Y,Z );   (* correct for precession and nutation  *)
    X:=X+VX; Y:=Y+VY; Z:= Z+VZ;  (* aberration                           *)
    POLAR( X,Y,Z, R,DEC,RA );    (* apparent right ascension,declination *)
  END;

(*-----------------------------------------------------------------------*)
(* ECLEQU: Conversion of ecliptic into equatorial coordinates            *)
(*         (T: equinox in Julian centuries since J2000)                  *)
(*-----------------------------------------------------------------------*)
PROCEDURE ECLEQU(T:extended;VAR X,Y,Z:extended);
  VAR EPS,C,S,V: extended;
  BEGIN
    EPS:=23.43929111-(46.8150+(0.00059-0.001813*T)*T)*T/3600.0;
    C:=CS(EPS);  S:=SN(EPS);
    V:=+C*Y-S*Z;  Z:=+S*Y+C*Z;  Y:=V;
  END;
(*-----------------------------------------------------------------------*)
(* ECLEQU: Conversion of ecliptic into equatorial coordinates            *)
(*         (T: equinox in Julian centuries since J2000)                  *)
(*-----------------------------------------------------------------------*)
PROCEDURE EQUECL(T:extended;VAR X,Y,Z:extended);
  VAR EPS,C,S,V: extended;
  BEGIN
    EPS:=23.43929111-(46.8150+(0.00059-0.001813*T)*T)*T/3600.0;
    C:=CS(EPS);  S:=SN(EPS);
    V:=+C*Y+S*Z;  Z:=-S*Y+C*Z;  Y:=V;
  END;
(*-----------------------------------------------------------------------*)
(* GAUSVEC: calculation of the Gaussian vectors (P,Q,R) from             *)
(*          ecliptic orbital elements:                                   *)
(*          LAN = longitude of the ascending node                        *)
(*          INC = inclination                                            *)
(*          AOP = argument of perihelion                                 *)
(*-----------------------------------------------------------------------*)
PROCEDURE GAUSVEC(LAN,INC,AOP:extended;VAR PQR:extended33);
  VAR C1,S1,C2,S2,C3,S3: extended;
  BEGIN
    C1:=CS(AOP);  C2:=CS(INC);  C3:=CS(LAN);
    S1:=SN(AOP);  S2:=SN(INC);  S3:=SN(LAN);
    PQR[1,1]:=+C1*C3-S1*C2*S3; PQR[1,2]:=-S1*C3-C1*C2*S3; PQR[1,3]:=+S2*S3;
    PQR[2,1]:=+C1*S3+S1*C2*C3; PQR[2,2]:=-S1*S3+C1*C2*C3; PQR[2,3]:=-S2*C3;
    PQR[3,1]:=+S1*S2;          PQR[3,2]:=+C1*S2;          PQR[3,3]:=+C2;
  END;
(*-----------------------------------------------------------------------*)
(* ORBECL: transformation of coordinates XX,YY referred to the           *)
(*         orbital plane into ecliptic coordinates X,Y,Z using           *)
(*         Gaussian vectors PQR                                          *)
(*-----------------------------------------------------------------------*)
PROCEDURE ORBECL(XX,YY:extended;PQR:extended33;VAR X,Y,Z:extended);
  BEGIN
    X:=PQR[1,1]*XX+PQR[1,2]*YY;
    Y:=PQR[2,1]*XX+PQR[2,2]*YY;
    Z:=PQR[3,1]*XX+PQR[3,2]*YY;
  END;
(*-----------------------------------------------------------------------*)
(* SITE:  calculates geocentric from geographic coordinates              *)
(*        RCPHI:  r * cos(phi') (geocentric; in earth radii)             *)
(*        RSPHI:  r * sin(phi') (geocentric; in earth radii)             *)
(*        PHI:    geographic latitude (deg)                              *)
(*-----------------------------------------------------------------------*)
PROCEDURE SITE ( PHI: extended; VAR RCPHI,RSPHI: extended );
  CONST E2 = 0.006694;         (* e**2=f(2-f) for flattening f=1/298.257 *)
  VAR   N,SNPHI: extended;
  BEGIN
    SNPHI := SN(PHI);    N := 1.0/SQRT(1.0-E2*SNPHI*SNPHI);
    RCPHI := N*CS(PHI);  RSPHI := (1.0-E2)*N*SNPHI;
  END;

(*-----------------------------------------------------------------------*)
(* STDEQU: transformation from standard coordinates into                 *)
(*         equatorial coordinates                                        *)
(*   RA0,DEC0: right ascension and declination of the optical axis (deg) *)
(*   XX,YY:    standard coordinates                                      *)
(*   RA,DEC:   right ascension and declination (deg)                     *)
(*-----------------------------------------------------------------------*)
PROCEDURE STDEQU ( RA0,DEC0,XX,YY: extended; VAR RA,DEC: extended);
  BEGIN
    RA  := RA0 + ATN ( -XX / (CS(DEC0)-YY*SN(DEC0)) );
    DEC := ASN ( (SN(DEC0)+YY*CS(DEC0))/SQRT(1.0+XX*XX+YY*YY) );
  END;

(*-----------------------------------------------------------------------*)
(* EQUSTD: transformation of equatorial coordinates into                 *)
(*         standard coordinates                                          *)
(*   RA0,DEC0: right ascension and declination of the optical axis (deg) *)
(*   RA,DEC:   right ascension and declination (deg)                     *)
(*   XX,YY:    standard coordinates                                      *)
(*-----------------------------------------------------------------------*)
PROCEDURE EQUSTD ( RA0,DEC0,RA,DEC: extended; VAR XX,YY: extended);
  VAR C: extended;
  BEGIN
    C  := CS(DEC0)*CS(DEC)*CS(RA-RA0)+SN(DEC0)*SN(DEC);
    XX := - ( CS(DEC)*SN(RA-RA0) ) / C;
    YY := - ( SN(DEC0)*CS(DEC)*CS(RA-RA0)-CS(DEC0)*SN(DEC) ) / C;
  END;

