(*-----------------------------------------------------------------------*)
(* T_FIT_SUN: approximates the equatorial coordinates of the             *)
(*            sun by Chebyshev expansions for a given period of time     *)
(*                                                                       *)
(*  TA     : first date (in julian centuries since J2000)                *)
(*  TB     : last date ( TB < TA + 1 year )                              *)
(*  N      : highest order                                               *)
(*  RA_POLY: coefficients for right ascencion                            *)
(*  DE_POLY: coefficients for declination                                *)
(*  R_POLY : coefficients for geocentric distance                        *)
(*-----------------------------------------------------------------------*)

PROCEDURE T_FIT_SUN ( TA,TB: extended; N: INTEGER;
                      VAR RA_POLY,DE_POLY,R_POLY: TPOLYNOM);
  CONST PI = 3.1415926535898;
        NDIM = 27;
  VAR   I,J,K          : INTEGER;
        FAC,BPA,BMA,PHI: extended;
        T,H,RA,DE,R    : ARRAY[0..NDIM] OF extended;
  BEGIN
    IF (NDIM<2*MAX_TP_DEG+1) THEN WRITELN(' NDIM too small in T_FIT_SUN');
    IF (N>MAX_TP_DEG) THEN WRITELN(' N too large in T_FIT_MOON');
    RA_POLY.M := N;    DE_POLY.M := N;    R_POLY.M := N;
    RA_POLY.A := TA;   DE_POLY.A := TA;   R_POLY.A := TA;
    RA_POLY.B := TB;   DE_POLY.B := TB;   R_POLY.B := TB;
    BMA := (TB-TA)/2.0;   BPA := (TB+TA)/2.0;
    FAC := 2.0/(N+1);
    PHI:=PI/(2*N+2);                              (* h(k)=cos(pi*k/N/2)  *)
    H[0]:=1.0; H[1]:=COS(PHI);
    FOR I:=2 TO (2*N+1) DO H[I]:=2*H[1]*H[I-1]-H[I-2];
    FOR K:=1 TO N+1 DO T[K] := H[2*K-1]*BMA+BPA;  (* subdivision points  *)
    FOR K:=1 TO N+1 DO SUNEQU(T[K],RA[K],DE[K],R[K]);
    FOR K := 2 TO N+1 DO                          (* make RA continuous  *)
      IF (RA[K-1]<RA[K]) THEN RA[K]:=RA[K]-360.0; (* in [-360,+360] deg  *)
    FOR J := 0 TO N DO                            (* calculate Chebyshev *)
      BEGIN                                       (* coefficients C(j)   *)
        PHI:=PI*J/(2*N+2); H[1]:=COS(PHI);
        FOR I:=2 TO (2*N+1) DO H[I] := 2*H[1]*H[I-1]-H[I-2];
        RA_POLY.C[J]:=0.0; DE_POLY.C[J]:=0.0; R_POLY.C[J]:=0.0;
        FOR K:=1 TO N+1 DO
          BEGIN
            RA_POLY.C[J] := RA_POLY.C[J] + H[2*K-1]*RA[K];
            DE_POLY.C[J] := DE_POLY.C[J] + H[2*K-1]*DE[K];
            R_POLY.C[J]  := R_POLY.C[J]  + H[2*K-1]*R[K];
          END;
        RA_POLY.C[J]:=RA_POLY.C[J]*FAC; DE_POLY.C[J]:=DE_POLY.C[J]*FAC;
        R_POLY.C[J] :=R_POLY.C[J]*FAC;
      END;
  END;
(*-----------------------------------------------------------------------*)
(* MINI_SUN: low precision solar coordinates (approx. 1')                *)
(*           T  : time in Julian centuries since J2000                   *)
(*                ( T=(JD-2452545)/36525 )                               *)
(*           RA : right ascension (in h; equinox of date)                *)
(*           DEC: declination (in deg; equinox of date)                  *)
(*-----------------------------------------------------------------------*)
PROCEDURE MINI_SUN(T:extended; VAR RA,DEC: extended);
  CONST P2  = 6.283185307; COSEPS=0.91748; SINEPS=0.39778;
  VAR   L,M,DL,SL,X,Y,Z,RHO: extended;
  FUNCTION FRAC(X:extended):extended;
    BEGIN  X:=X-TRUNC(X); IF (X<0) THEN X:=X+1; FRAC:=X  END;
  BEGIN 
    M  := P2*FRAC(0.993133+99.997361*T); 
    DL := 6893.0*SIN(M)+72.0*SIN(2*M);
    L  := P2*FRAC(0.7859453 + M/P2 + (6191.2*T+DL)/1296E3);
    SL := SIN(L);
    X:=COS(L); Y:=COSEPS*SL; Z:=SINEPS*SL; RHO:=SQRT(1.0-Z*Z);
    DEC := (360.0/P2)*ARCTAN(Z/RHO); 
    RA  := ( 48.0/P2)*ARCTAN(Y/(X+RHO)); IF (RA<0) THEN RA:=RA+24.0;
  END;

(*-----------------------------------------------------------------------*)
(* SUN200: ecliptic coordinates L,B,R (in deg and AU) of the             *)
(*         Sun referred to the mean equinox of date                      *)
(*         (T: time in Julian centuries since J2000)                     *)
(*         (   = (JED-2451545.0)/36525             )                     *)
(*-----------------------------------------------------------------------*)
PROCEDURE SUN200(T:extended;VAR L,B,R:extended);
  CONST P2=6.283185307;
  VAR C3,S3:          ARRAY [-1..7] OF extended;
      C,S:            ARRAY [-8..0] OF extended;
      M2,M3,M4,M5,M6: extended;
      D,A,UU:         extended;
      U,V,DL,DR,DB:   extended;
      I:              INTEGER;

  FUNCTION FRAC(X:extended):extended;
    (* for some compilers TRUNC has to be replaced by LONG_TRUNC *)
    (* or INT (Turbo-Pascal) if the routine is used with T<-24   *)
    BEGIN  X:=X-TRUNC(X); IF (X<0) THEN X:=X+1.0; FRAC:=X  END;

  PROCEDURE ADDTHE(C1,S1,C2,S2:extended; VAR C,S:extended);
    BEGIN  C:=C1*C2-S1*S2; S:=S1*C2+C1*S2; END;

  PROCEDURE TERM(I1,I,IT:INTEGER;DLC,DLS,DRC,DRS,DBC,DBS:extended);
    BEGIN
      IF IT=0 THEN ADDTHE(C3[I1],S3[I1],C[I],S[I],U,V)
              ELSE BEGIN U:=U*T; V:=V*T END;
      DL:=DL+DLC*U+DLS*V; DR:=DR+DRC*U+DRS*V; DB:=DB+DBC*U+DBS*V;
    END;


  PROCEDURE PERTVEN;  (* Keplerian terms and perturbations by Venus *)
    VAR I: INTEGER;
    BEGIN
      C[0]:=1.0; S[0]:=0.0; C[-1]:=COS(M2); S[-1]:=-SIN(M2);
      FOR I:=-1 DOWNTO -5 DO ADDTHE(C[I],S[I],C[-1],S[-1],C[I-1],S[I-1]);
      TERM(1, 0,0,-0.22,6892.76,-16707.37, -0.54, 0.00, 0.00);
      TERM(1, 0,1,-0.06, -17.35,    42.04, -0.15, 0.00, 0.00);
      TERM(1, 0,2,-0.01,  -0.05,     0.13, -0.02, 0.00, 0.00);
      TERM(2, 0,0, 0.00,  71.98,  -139.57,  0.00, 0.00, 0.00);
      TERM(2, 0,1, 0.00,  -0.36,     0.70,  0.00, 0.00, 0.00);
      TERM(3, 0,0, 0.00,   1.04,    -1.75,  0.00, 0.00, 0.00);
      TERM(0,-1,0, 0.03,  -0.07,    -0.16, -0.07, 0.02,-0.02);
      TERM(1,-1,0, 2.35,  -4.23,    -4.75, -2.64, 0.00, 0.00);
      TERM(1,-2,0,-0.10,   0.06,     0.12,  0.20, 0.02, 0.00);
      TERM(2,-1,0,-0.06,  -0.03,     0.20, -0.01, 0.01,-0.09);
      TERM(2,-2,0,-4.70,   2.90,     8.28, 13.42, 0.01,-0.01);
      TERM(3,-2,0, 1.80,  -1.74,    -1.44, -1.57, 0.04,-0.06);
      TERM(3,-3,0,-0.67,   0.03,     0.11,  2.43, 0.01, 0.00);
      TERM(4,-2,0, 0.03,  -0.03,     0.10,  0.09, 0.01,-0.01);
      TERM(4,-3,0, 1.51,  -0.40,    -0.88, -3.36, 0.18,-0.10);
      TERM(4,-4,0,-0.19,  -0.09,    -0.38,  0.77, 0.00, 0.00);
      TERM(5,-3,0, 0.76,  -0.68,     0.30,  0.37, 0.01, 0.00);
      TERM(5,-4,0,-0.14,  -0.04,    -0.11,  0.43,-0.03, 0.00);
      TERM(5,-5,0,-0.05,  -0.07,    -0.31,  0.21, 0.00, 0.00);
      TERM(6,-4,0, 0.15,  -0.04,    -0.06, -0.21, 0.01, 0.00);
      TERM(6,-5,0,-0.03,  -0.03,    -0.09,  0.09,-0.01, 0.00);
      TERM(6,-6,0, 0.00,  -0.04,    -0.18,  0.02, 0.00, 0.00);
      TERM(7,-5,0,-0.12,  -0.03,    -0.08,  0.31,-0.02,-0.01);
    END;

  PROCEDURE PERTMAR;  (* perturbations by Mars *)
    VAR I: INTEGER;
    BEGIN
      C[-1]:=COS(M4); S[-1]:=-SIN(M4);
      FOR I:=-1 DOWNTO -7 DO ADDTHE(C[I],S[I],C[-1],S[-1],C[I-1],S[I-1]);
      TERM(1,-1,0,-0.22,   0.17,    -0.21, -0.27, 0.00, 0.00);
      TERM(1,-2,0,-1.66,   0.62,     0.16,  0.28, 0.00, 0.00);
      TERM(2,-2,0, 1.96,   0.57,    -1.32,  4.55, 0.00, 0.01);
      TERM(2,-3,0, 0.40,   0.15,    -0.17,  0.46, 0.00, 0.00);
      TERM(2,-4,0, 0.53,   0.26,     0.09, -0.22, 0.00, 0.00);
      TERM(3,-3,0, 0.05,   0.12,    -0.35,  0.15, 0.00, 0.00);
      TERM(3,-4,0,-0.13,  -0.48,     1.06, -0.29, 0.01, 0.00);
      TERM(3,-5,0,-0.04,  -0.20,     0.20, -0.04, 0.00, 0.00);
      TERM(4,-4,0, 0.00,  -0.03,     0.10,  0.04, 0.00, 0.00);
      TERM(4,-5,0, 0.05,  -0.07,     0.20,  0.14, 0.00, 0.00);
      TERM(4,-6,0,-0.10,   0.11,    -0.23, -0.22, 0.00, 0.00);
      TERM(5,-7,0,-0.05,   0.00,     0.01, -0.14, 0.00, 0.00);
      TERM(5,-8,0, 0.05,   0.01,    -0.02,  0.10, 0.00, 0.00);
    END;

  PROCEDURE PERTJUP;  (* perturbations by Jupiter *)
    VAR I: INTEGER;
    BEGIN
      C[-1]:=COS(M5); S[-1]:=-SIN(M5);
      FOR I:=-1 DOWNTO -3 DO ADDTHE(C[I],S[I],C[-1],S[-1],C[I-1],S[I-1]);
      TERM(1,-1,0, 0.01,   0.07,     0.18, -0.02, 0.00,-0.02);
      TERM(0,-1,0,-0.31,   2.58,     0.52,  0.34, 0.02, 0.00);
      TERM(1,-1,0,-7.21,  -0.06,     0.13,-16.27, 0.00,-0.02);
      TERM(1,-2,0,-0.54,  -1.52,     3.09, -1.12, 0.01,-0.17);
      TERM(1,-3,0,-0.03,  -0.21,     0.38, -0.06, 0.00,-0.02);
      TERM(2,-1,0,-0.16,   0.05,    -0.18, -0.31, 0.01, 0.00);
      TERM(2,-2,0, 0.14,  -2.73,     9.23,  0.48, 0.00, 0.00);
      TERM(2,-3,0, 0.07,  -0.55,     1.83,  0.25, 0.01, 0.00);
      TERM(2,-4,0, 0.02,  -0.08,     0.25,  0.06, 0.00, 0.00);
      TERM(3,-2,0, 0.01,  -0.07,     0.16,  0.04, 0.00, 0.00);
      TERM(3,-3,0,-0.16,  -0.03,     0.08, -0.64, 0.00, 0.00);
      TERM(3,-4,0,-0.04,  -0.01,     0.03, -0.17, 0.00, 0.00);
    END;

  PROCEDURE PERTSAT;  (* perturbations by Saturn *)
    BEGIN
      C[-1]:=COS(M6); S[-1]:=-SIN(M6);
      ADDTHE(C[-1],S[-1],C[-1],S[-1],C[-2],S[-2]);
      TERM(0,-1,0, 0.00,   0.32,     0.01,  0.00, 0.00, 0.00);
      TERM(1,-1,0,-0.08,  -0.41,     0.97, -0.18, 0.00,-0.01);
      TERM(1,-2,0, 0.04,   0.10,    -0.23,  0.10, 0.00, 0.00);
      TERM(2,-2,0, 0.04,   0.10,    -0.35,  0.13, 0.00, 0.00);
    END;

  PROCEDURE PERTMOO;  (* difference between the Earth-Moon      *)
    BEGIN             (* barycenter and the center of the Earth *)
      DL := DL +  6.45*SIN(D) - 0.42*SIN(D-A) + 0.18*SIN(D+A)
                              + 0.17*SIN(D-M3) - 0.06*SIN(D+M3);
      DR := DR + 30.76*COS(D) - 3.06*COS(D-A)+ 0.85*COS(D+A)
                              - 0.58*COS(D+M3) + 0.57*COS(D-M3);
      DB := DB + 0.576*SIN(UU);
    END;

  BEGIN  (* SUN200 *)

    DL:=0.0; DR:=0.0; DB:=0.0;
    M2:=P2*FRAC(0.1387306+162.5485917*T);
    M3:=P2*FRAC(0.9931266+99.9973604*T);
    M4:=P2*FRAC(0.0543250+ 53.1666028*T);
    M5:=P2*FRAC(0.0551750+ 8.4293972*T);
    M6:=P2*FRAC(0.8816500+  3.3938722*T); D :=P2*FRAC(0.8274+1236.8531*T);
    A :=P2*FRAC(0.3749+1325.5524*T);      UU:=P2*FRAC(0.2591+1342.2278*T);
    C3[0]:=1.0;     S3[0]:=0.0;
    C3[1]:=COS(M3); S3[1]:=SIN(M3);  C3[-1]:=C3[1]; S3[-1]:=-S3[1];
    FOR I:=2 TO 7 DO ADDTHE(C3[I-1],S3[I-1],C3[1],S3[1],C3[I],S3[I]);
    PERTVEN; PERTMAR; PERTJUP; PERTSAT; PERTMOO;
    DL:=DL + 6.40*SIN(P2*(0.6983+0.0561*T))+1.87*SIN(P2*(0.5764+0.4174*T))
           + 0.27*SIN(P2*(0.4189+0.3306*T))+0.20*SIN(P2*(0.3581+2.4814*T));
    L:= 360.0*FRAC(0.7859453 + M3/P2 + ((6191.2+1.1*T)*T+DL)/1296.0E3 );
    R:= 1.0001398 - 0.0000007*T  +  DR*1E-6;
    B:= DB/3600.0;

  END;   (* SUN200 *)

(*-----------------------------------------------------------------------*)
(* SUNEQU: apparent equatorial coordinates of the Sun                    *)
(*         (right ascension RA, declination DEC in deg, R in AU)         *)
(*         T in Julian centuries since J2000 (T=(JD-2451545.0)/36525)    *)
(*-----------------------------------------------------------------------*)
PROCEDURE SUNEQU(T:extended;VAR RA,DEC,R:extended);
  VAR DT,L,B,X,Y,Z: extended;
  BEGIN
    DT := (8.32/1440.0)/36525.0; (* light-time correction of 8.32 min    *)
    SUN200(T-DT,L,B,R);          (* geocentric ecliptic coordinates      *)
    CART(R,B,L,X,Y,Z);           (* cartesian ecliptic coordinates       *)
    ECLEQU(T,X,Y,Z);             (* equatorial ecliptic coordinates      *)
    NUTEQU(T,X,Y,Z);             (* correction for nutation              *)
    POLAR(X,Y,Z,R,DEC,RA);       (* spherical coordinates;               *)
  END;                           (* true equinox of date                 *)
